@@ -33,15 +33,18 @@ pref("helpers.private_mailcap_file", "");
 pref("helpers.global_mime_types_file", "");
 pref("helpers.private_mime_types_file", "");
 
-// use google for keywords
-pref("keyword.enabled", true);
+// disable keyword searches by default, see bug #158486
+pref("keyword.enabled", false);
 
 // disable usless security warnings
 pref("security.warn_entering_secure", false);
 pref("security.warn_entering_secure.show_once", true);
 pref("security.warn_leaving_secure", false);
+pref("security.warn_leaving_secure.show_once", false);
 pref("security.warn_submit_insecure", false);
 pref("security.warn_submit_insecure.show_once", false);
+pref("security.warn_viewing_mixed", true);
+pref("security.warn_viewing_mixed.show_once", false);
 
 // fonts
 pref("font.size.unit", "pt");
