@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.cpp,v 1.86 2005/08/08 13:39:50 chpe Exp $
+ *  $Id: ContentHandler.cpp,v 1.88 2005/08/13 18:51:05 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -330,7 +330,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction ()
 			(GTK_MESSAGE_DIALOG (dialog),
 			/* translators: First %s is the file type description,
 			   Second %s is the file name */
-			_("File Type: %s.\n\nIt is unsafe to open \"%s\", "
+			_("File Type: %s.\n\nIt is unsafe to open \"%s\" as "
 			  "it could potentially damage your documents or "
 			  "invade your privacy. You can download it instead."),
 			  mime_description, file_name.get());		
@@ -411,6 +411,19 @@ NS_METHOD GContentHandler::MIMEInitiateAction (void)
 	mPermission = ephy_file_check_mime (mMimeType);
 #endif
 
+	/* HACK! Check that this 'helper application' isn't Epiphany itself,
+	 * see bug #310023.
+	 */
+	if (mHelperApp)
+	{
+		const char *id = gnome_vfs_mime_application_get_desktop_id (mHelperApp);
+
+		if (id && strcmp (id, "epiphany.desktop") == 0)
+		{
+			mHelperApp = nsnull;
+		}
+	}
+
 	if (auto_downloads)
 	{
 		mAction = CONTENT_ACTION_OPEN;
