@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: egg-toolbars-model.c,v 1.34 2005/01/10 00:14:23 chpe Exp $
+ *  $Id: egg-toolbars-model.c,v 1.35 2005/06/01 17:19:10 chpe Exp $
  */
 
 #include "config.h"
@@ -113,16 +113,16 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
   tl = t->priv->toolbars;
 
   xmlIndentTreeOutput = TRUE;
-  doc = xmlNewDoc ("1.0");
-  doc->children = xmlNewDocNode (doc, NULL, "toolbars", NULL);
+  doc = xmlNewDoc ((const xmlChar*) "1.0");
+  doc->children = xmlNewDocNode (doc, NULL, (const xmlChar*) "toolbars", NULL);
 
   for (l1 = tl->children; l1 != NULL; l1 = l1->next)
     {
       xmlNodePtr tnode;
       EggToolbarsToolbar *toolbar = l1->data;
 
-      tnode = xmlNewChild (doc->children, NULL, "toolbar", NULL);
-      xmlSetProp (tnode, "name", toolbar->name);
+      tnode = xmlNewChild (doc->children, NULL, (const xmlChar*) "toolbar", NULL);
+      xmlSetProp (tnode, (const xmlChar*) "name", (const xmlChar*) toolbar->name);
 
       for (l2 = l1->children; l2 != NULL; l2 = l2->next)
 	{
@@ -131,16 +131,16 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
 
 	  if (item->separator)
 	    {
-	      node = xmlNewChild (tnode, NULL, "separator", NULL);
+	      node = xmlNewChild (tnode, NULL, (const xmlChar*) "separator", NULL);
 	    }
 	  else
 	    {
 	      char *data;
 
-	      node = xmlNewChild (tnode, NULL, "toolitem", NULL);
+	      node = xmlNewChild (tnode, NULL, (const xmlChar*) "toolitem", NULL);
 	      data = egg_toolbars_model_get_item_data (t, item->type, item->id);
-	      xmlSetProp (node, "type", item->type);
-	      xmlSetProp (node, "name", data);
+	      xmlSetProp (node, (const xmlChar*) "type", (const xmlChar*) item->type);
+	      xmlSetProp (node, (const xmlChar*) "name", (const xmlChar*) data);
 	      g_free (data);
 	    }
 	}
@@ -217,7 +217,7 @@ egg_toolbars_model_save (EggToolbarsModel *t,
 
   doc = egg_toolbars_model_to_xml (t);
   root = xmlDocGetRootElement (doc);
-  xmlSetProp (root, "version", version);
+  xmlSetProp (root, (const xmlChar*) "version", (const xmlChar*) version);
   safe_save_xml (xml_file, doc);
   xmlFreeDoc (doc);
 }
@@ -368,31 +368,31 @@ parse_item_list (EggToolbarsModel *t,
 {
   while (child)
     {
-      if (xmlStrEqual (child->name, "toolitem"))
+      if (xmlStrEqual (child->name, (const xmlChar*) "toolitem"))
 	{
 	  xmlChar *name, *type;
 	  char *id;
 
-	  name = xmlGetProp (child, "name");
-	  type = xmlGetProp (child, "type");
+	  name = xmlGetProp (child, (const xmlChar*) "name");
+	  type = xmlGetProp (child, (const xmlChar*) "type");
           if (type == NULL)
             {
-              type = xmlStrdup (EGG_TOOLBAR_ITEM_TYPE);
+              type = xmlCharStrdup (EGG_TOOLBAR_ITEM_TYPE);
             }
 
 	  if (name != NULL && name[0] != '\0' && type != NULL)
 	    {
-              id = egg_toolbars_model_get_item_id (t, type, name);
+              id = egg_toolbars_model_get_item_id (t, (const char*)type, (const char*)name);
 	      if (id != NULL)
 	        {
-	          egg_toolbars_model_add_item (t, position, -1, id, type);
+	          egg_toolbars_model_add_item (t, position, -1, id, (const char*)type);
                 }
               g_free (id);
             }
 	  xmlFree (name);
           xmlFree (type);
 	}
-      else if (xmlStrEqual (child->name, "separator"))
+      else if (xmlStrEqual (child->name, (const xmlChar*) "separator"))
 	{
 	  egg_toolbars_model_add_separator (t, position, -1);
 	}
@@ -428,18 +428,18 @@ parse_toolbars (EggToolbarsModel *t,
 {
   while (child)
     {
-      if (xmlStrEqual (child->name, "toolbar"))
+      if (xmlStrEqual (child->name, (const xmlChar*) "toolbar"))
 	{
 	  xmlChar *name;
 	  xmlChar *style;
 	  int position;
 
-	  name = xmlGetProp (child, "name");
-	  position = egg_toolbars_model_add_toolbar (t, -1, name);
+	  name = xmlGetProp (child, (const xmlChar*) "name");
+	  position = egg_toolbars_model_add_toolbar (t, -1, (const char*) name);
 	  xmlFree (name);
 
-	  style = xmlGetProp (child, "style");
-	  if (style && xmlStrEqual (style, "icons-only"))
+	  style = xmlGetProp (child, (const xmlChar*) "style");
+	  if (style && xmlStrEqual (style, (const xmlChar*) "icons-only"))
 	    {
 	      /* FIXME: use toolbar position instead of 0 */
 	      egg_toolbars_model_set_flags (t, 0, EGG_TB_MODEL_ICONS);
