@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.h,v 1.33 2005/07/10 20:57:14 chpe Exp $
+ *  $Id: ephy-bookmarks.h,v 1.35 2005/07/26 17:41:04 chpe Exp $
  */
 
 #ifndef EPHY_BOOKMARKS_H
@@ -66,6 +66,9 @@ struct _EphyBookmarksClass
 	GObjectClass parent_class;
 
 	void	(* tree_changed)	(EphyBookmarks *eb);
+	char *	(* resolve_address)	(EphyBookmarks *eb,
+					 const char *address,
+					 const char *argument);
 };
 
 GType		  ephy_bookmarks_get_type		(void);
@@ -94,9 +97,11 @@ void		  ephy_bookmarks_set_address    	(EphyBookmarks *eb,
 							 EphyNode *bookmark,
 							 const char *address);
 
-char		 *ephy_bookmarks_solve_smart_url	 (EphyBookmarks *eb,
-							 const char *smart_url,
-							 const char *content);
+char		 *ephy_bookmarks_resolve_address	(EphyBookmarks *eb,
+							 const char *address,
+							 const char *parameter);
+
+guint		ephy_bookmarks_get_smart_bookmark_width (EphyNode *bookmark);
 
 GtkWidget      *ephy_bookmarks_show_bookmark_properties (EphyBookmarks *bookmarks,
 							 EphyNode *bookmark,
