@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-lockdown.c,v 1.2 2005/05/07 11:05:53 chpe Exp $
+ *  $Id: ephy-lockdown.c,v 1.5 2005/08/10 18:14:09 chpe Exp $
  */
 
 #include "config.h"
@@ -107,8 +107,9 @@ update_location_editable (EphyWindow *window,
 			g_return_if_fail (embed != NULL);
 
 			address = ephy_embed_get_location (embed, TRUE);
-			ephy_toolbar_set_location (EPHY_TOOLBAR (toolbar), address);
-			ephy_tab_set_location (tab, address, EPHY_TAB_ADDRESS_EXPIRE_CURRENT);
+			ephy_toolbar_set_location (EPHY_TOOLBAR (toolbar), address, NULL);
+			ephy_tab_set_typed_address (tab, NULL,
+						    EPHY_TAB_ADDRESS_EXPIRE_CURRENT);
 			g_free (address);
 		}
 	}
